// test-qtl.cpp : ̨Ӧóڵ㡣
//

#ifndef WIN32
#define OutputDebugString printf
#include <iostream>
#include <algorithm>
using namespace std; 
#else 
#include "stdafx.h"
#endif 

#include <time.h>
#include <sstream>
#include <string>
#include "qtl/qtl_sqlite.hpp"

#define SQLITE_TIMEOUT 180000  // sqlite timeout
#define POPBOX_MSG_RETRY_MAX 5 // resend retry max
//#define HAS_MEMBER_PRINT

uint64_t test_insert_single(qtl::sqlite::database &db)
{
	time_t now = time(0); 
	srand(now); 

	int tmp = rand() % 1000; 
	uint64_t id = db.insert_direct("insert into popbox_msg(msgid, msgtype, appname, uid, status, count, msgbody, stamp) values(?, ?, ?, ?, ?, ?, ?, ?)", 
		std::to_string(tmp), 108, "GDraw", "1923374929399", 1, 0, "this is msgbody", now);

	printf("insert record with msgid %d return %d\n", tmp, (int)id); 
	return id; 
}

void test_update_single(qtl::sqlite::database &db, uint64_t rowid)
{
	time_t now = time(0); 
	uint64_t affected = 0; 
	db.execute_direct("update popbox_msg set status=?, count=?, stamp=? where rowid=?", &affected, 0, 3, now, (int)rowid);
	//db.execute("update popbox_msg set status=0 where msgid=?", id, &affected); 
	printf("update record with rowid %d affected %d records\n", (int)rowid, (int)affected);
}

void test_insert_multi(qtl::sqlite::database &db)
{
	uint64_t affected = 0;
	int tmp[3] = { 0 }; 
	for (int i=0; i<3; ++i)
		tmp[i] = rand() % 1000;
 
	auto stmt = db.open_command("insert into popbox_msg(msgid, msgtype, appname, uid, status, count, msgbody, stamp) "
		"values(?, 108, 'GDraw', '1923374929399', 1, 0, 'this is msgbody', strftime('%s','now'))");
	qtl::execute(stmt, &affected, std::to_string(tmp[0]), std::to_string(tmp[1]), std::to_string(tmp[2])); 
	printf("insert %d record\n", (int)affected); 
}

void test_update_multi(qtl::sqlite::database &db)
{
	uint64_t affected = 0;
	int id[3] = { 19, 20, 21 }; 

	auto stmt = db.open_command("update popbox_msg set status=0, count=2, stamp=strftime('%s','now') where rowid=? "); 
	qtl::execute(stmt, &affected, id[0], id[1], id[2]);
	printf("update %d record\n", (int)affected);
}

void test_delete(qtl::sqlite::database &db)
{
	uint64_t affected = 0;
	db.execute_direct("delete from popbox_msg where msgtype=? and appname=? and uid=?", &affected, 108, "GDraw", "1923374929399");
	printf("delete record affected %d rows\n", (int)affected);
}

void test_query_single(qtl::sqlite::database &db, uint64_t rowid)
{
	std::string msg; 
#if 1
	db.query_first("select msgbody from popbox_msg where rowid=?", (int)rowid, msg);
#else
	db.query_first("select msgbody from popbox_msg", msg); 
#endif
	printf("row %d: %s\n", (int)rowid, msg.c_str());
}

void test_query_single_ex(qtl::sqlite::database &db, uint64_t rowid)
{
	time_t stamp = 0;
	int status = 0, count = 0; 

	std::ostringstream oss; 
	oss << "select status, count, stamp from popbox_msg where rowid=" << rowid; 
	db.query_first_direct(oss.str (), status, count, stamp);
	printf("row %d: status %d, count %d, stamp %d\n", (int)rowid, status, count, (int)stamp);
}

void test_query_multi(qtl::sqlite::database &db)
{
	int cnt = 0; 
#if 1
	db.query("select status, count, stamp from popbox_msg where appname=?", "GDraw", 
		[&cnt](int status, int count, time_t stamp){
		printf("%d, %d, %d\n", status, count, (int)stamp); 
		cnt++; 
	}); 
#else
	db.query("select msgid, msgtype, appname, uid, status, count, msgbody, stamp from popbox_msg where appname=?", "GDraw",
		[&cnt](std::string const& msgid, int msgtype, std::string const& appname, std::string const& uid, int status, int count, std::string const& msgbody, time_t stamp){
		printf("%s, %d, %s, %s, %d, %d, %s, %d\n", msgid.c_str(), msgtype, appname.c_str(), uid.c_str(), status, count, msgbody.c_str(), (int)stamp);
		cnt++;
	});
#endif

	printf("query %d records\n", cnt); 
}

// for srvmsgs
class popbox_msg_t
{
public:
	void dump(char const* prompt) const;
#ifdef HAS_MEMBER_PRINT
	void print(); 
#endif

	int msgtype = 0;   // 108 or 402
	int status = 0;    // send to server result, (1:ok; 0:fail)
	int count = 0;     // retry times, if exceed POPBOX_MSG_RETRY_MAX, stop retry
	time_t stamp = 0;  // receive time
	std::string msgid;
	std::string msgbody;
	std::string appname;
	std::string uid;
};


void popbox_msg_t::dump(char const* prompt) const
{
	tm* t = localtime(&stamp);
	printf("%s : %s,%s,%s,%d,%d,%d, %04d-%02d-%02d %02d:%02d:%02d, %s\n",
		prompt,
		appname.c_str(),
		uid.c_str(),
		msgid.c_str(),
		msgtype,
		status,
		count,
		t->tm_year + 1900,
		t->tm_mon + 1,
		t->tm_mday + 1,
		t->tm_hour,
		t->tm_min,
		t->tm_sec,
		msgbody.c_str());
}

#ifdef HAS_MEMBER_PRINT
void popbox_msg_t::print ()
{
	dump("msg"); 
}
#endif

namespace qtl
{
	template<>
	inline void bind_record<qtl::sqlite::statement, popbox_msg_t>(qtl::sqlite::statement& command, popbox_msg_t&& v)
	{
#if 1
		int n = 0;
		qtl::bind_field(command, n++, v.msgid);
		qtl::bind_field(command, n++, v.msgtype);
		qtl::bind_field(command, n++, v.appname);
		qtl::bind_field(command, n++, v.uid);
		qtl::bind_field(command, n++, v.status);
		qtl::bind_field(command, n++, v.count);
		qtl::bind_field(command, n++, v.msgbody);
		qtl::bind_field(command, n++, v.stamp);
#else
		qtl::bind_fields(command, v.msgid, v.msgtype, v.appname, v.uid, v.status, v.count, v.msgbody, v.stamp); 
#endif
	}
}

void test_query_multi_ex(qtl::sqlite::database &db)
{
	int cnt = 0;
#ifdef HAS_MEMBER_PRINT
	db.query("select msgid, msgtype, appname, uid, status, count, msgbody, stamp from popbox_msg where appname=?", "GDraw",
		&popbox_msg_t::print);
#else
	db.query("select msgid, msgtype, appname, uid, status, count, msgbody, stamp from popbox_msg where appname=?", "GDraw", 
		[&cnt](popbox_msg_t const& pm){
		pm.dump("msg"); 
		cnt++; 
	});
#endif

	printf("query %d records\n", cnt);
}

void test_query_multi_ul(qtl::sqlite::database &db)
{
	int cnt = 0;
	for(auto& pm : db.result<popbox_msg_t>("select msgid, msgtype, appname, uid, status, count, msgbody, stamp from popbox_msg where appname=?", "GDraw"))
	{
		pm.dump("msg");
		cnt++;
	}

	printf("query %d records\n", cnt);
}

#if 0
void my_bind(popbox_msg_t&& v, qtl::sqlite::statement& command)
{
	int n = 0; 
	qtl::bind_field(command, n++, v.status);
	qtl::bind_field(command, n++, v.count);
	qtl::bind_field(command, n++, v.stamp);
}

void test_query_multi_custom(qtl::sqlite::database &db)
{
	int cnt = 0;
	db.query_explicit("select status, count, stamp from popbox_msg where appname=?", "GDraw", 
		qtl::custom_bind(popbox_msg_t(), my_bind), 
		[&cnt](popbox_msg_t const& pm){
		printf("msg: %d, %d, %d\n", pm.status, pm.count, pm.stamp); 
		cnt++; 
	});

	printf("query %d records\n", cnt);
}
#endif 

int main(int argc, char* argv[])
{
	int ret = 0;
	srand(time(0));
	uint64_t rowid = 0; 
	qtl::sqlite::database db(SQLITE_TIMEOUT);

	try
	{
		// copy of gcm.db, and create following table:
		// create table popmsg (msgid text not null, msgtype integer not null, cid text not null, uid text not null, 
		// status integer not null, count integer not null, msgbody text not null, stamp timestamp not null, 
		// primary key (msgid, msgtype, cid, uid));
		db.open("../data/gcm.db", NULL);
		printf("open db OK\n");

#if 1
		rowid = test_insert_single(db);
#endif 

#if 1
		test_update_single(db, rowid);
#endif 

#if 1
		test_insert_multi(db); 
#endif 

#if 1
		test_update_multi(db); 
#endif

#if 1
		test_query_single(db, rowid); 
#endif 

#if 1
		test_query_single_ex(db, rowid); 
#endif 

#if 1
		test_query_multi(db); 
#endif 

#if 1
		test_query_multi_ex(db); 
#endif 

#if 1
		test_query_multi_ul(db); 
#endif 

		//test_query_multi_custom(db); 
		
#if 1
		test_delete(db);
#endif 

		db.close();
	}
	catch (qtl::sqlite::error &e)
	{
		printf("manipute db error %d: %s\n", e.code(), e.what());
		db.close();
		return -1;
	}

    return 0; 
}
